// SPDX-License-Identifier: GPL-2.0
/*
 * sc8562_scp.c
 *
 * sc8562 scp driver
 *
 * Copyright (c) 2022-2022 Huawei Technologies Co., Ltd.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 */

#include "sc8562_scp.h"
#include "sc8562_i2c.h"
#include <linux/mutex.h>
#include <chipset_common/hwpower/common_module/power_delay.h>
#include <chipset_common/hwpower/common_module/power_printk.h>
#include <chipset_common/hwpower/protocol/adapter_protocol.h>
#include <chipset_common/hwpower/protocol/adapter_protocol_scp.h>
#include <chipset_common/hwpower/protocol/adapter_protocol_fcp.h>

#define HWLOG_TAG sc8562_scp
HWLOG_REGIST();


static bool sc8562_scp_data_valid(struct sc8562_device_info *di)
{
	int ret, i;
	u8 scp_stat = 0;
	u8 fifo_stat = 0;
	u8 data_num;

	ret = sc8562_read_byte(di, SC8562_SCP_STAT_REG, &scp_stat);
	ret += sc8562_read_byte(di, SC8562_SCP_FIFO_STAT_REG, &fifo_stat);
	if (ret)
		return false;

	data_num = fifo_stat & SC8562_RX_FIFO_CNT_STAT_MASK;
	for (i = 0; i < data_num; i++) {
		(void)sc8562_read_byte(di, SC8562_SCP_RDATA_REG, &di->scp_data[i]);
		hwlog_debug("ic_%u read scp_data=0x%x\n", di->ic_role, di->scp_data[i]);
	}
	hwlog_info("ic_%u scp_stat=0x%x,fifo_stat=0x%x,rx_num=%d,scp_op_num=%d,scp_op=%d\n",
		di->ic_role, scp_stat, fifo_stat, data_num, di->scp_op_num, di->scp_op);

	/* first scp data should be ack(0x08 or 0x88) */
	if (((di->scp_data[0] & 0x0F) == SC8562_SCP_ACK) &&
		(scp_stat == SC8562_SCP_NO_ERR) &&
		(((di->scp_op == SC8562_SCP_WRITE_OP) &&
		(data_num == SC8562_SCP_ACK_AND_CRC_LEN)) ||
		((di->scp_op == SC8562_SCP_READ_OP) &&
		(data_num == di->scp_op_num + SC8562_SCP_ACK_AND_CRC_LEN))))
		return true;

	return false;
}

static int sc8562_scp_cmd_transfer_check(struct sc8562_device_info *di)
{
	int cnt = 0;

	do {
		(void)power_msleep(DT_MSLEEP_50MS, 0, NULL);
		if (di->scp_trans_done) {
			if (sc8562_scp_data_valid(di)) {
				hwlog_debug("ic_%u scp_trans success\n", di->ic_role);
				return 0;
			}
			hwlog_err("ic_%u scp_trans_done, but data err\n", di->ic_role);
			return -EINVAL;
		}
		cnt++;
	} while (cnt < SC8562_SCP_ACK_RETRY_TIME);

	hwlog_err("ic_%u scp adapter trans time out\n", di->ic_role);
	return -EINVAL;
}

static int sc8562_scp_adapter_reg_read(u8 *val, u8 reg, struct sc8562_device_info *di)
{
	int ret, i;

	mutex_lock(&di->accp_adapter_reg_lock);

	/* clear scp data */
	memset(di->scp_data, 0, sizeof(di->scp_data));

	di->scp_op = SC8562_SCP_READ_OP;
	di->scp_op_num = SC8562_SCP_SBRWR_NUM;
	di->scp_trans_done = false;
	for (i = 0; i < SC8562_SCP_RETRY_TIME; i++) {
		/* clear tx/rx fifo */
		ret = sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
			SC8562_SCP_CTRL_CLR_TX_FIFO_MASK, SC8562_SCP_CTRL_CLR_TX_FIFO_SHIFT, 1);
		ret += sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
			SC8562_SCP_CTRL_CLR_RX_FIFO_MASK, SC8562_SCP_CTRL_CLR_RX_FIFO_SHIFT, 1);
		/* write data */
		ret += sc8562_write_byte(di, SC8562_SCP_WDATA_REG,
			SC8562_SCP_CMD_SBRRD);
		ret += sc8562_write_byte(di, SC8562_SCP_WDATA_REG, reg);
		/* start trans */
		ret += sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
			SC8562_SCP_CTRL_SND_TRANS_MASK,
			SC8562_SCP_CTRL_SND_TRANS_SHIFT, 1);
		if (ret) {
			mutex_unlock(&di->accp_adapter_reg_lock);
			return -EIO;
		}
		/* check cmd transfer success or fail */
		if (sc8562_scp_cmd_transfer_check(di) == 0) {
			memcpy(val, &di->scp_data[1], di->scp_op_num);
			break;
		}
	}
	if (i >= SC8562_SCP_RETRY_TIME) {
		hwlog_err("ic_%u ack error, retry\n", di->ic_role);
		ret = -EINVAL;
	}

	mutex_unlock(&di->accp_adapter_reg_lock);
	return ret;
}

static int sc8562_scp_adapter_multi_reg_read(u8 reg, u8 *val,
	u8 num, struct sc8562_device_info *di)
{
	int ret, i;

	mutex_lock(&di->accp_adapter_reg_lock);

	/* clear scp data */
	memset(di->scp_data, 0, sizeof(di->scp_data));

	di->scp_op = SC8562_SCP_READ_OP;
	di->scp_op_num = num;
	di->scp_trans_done = false;
	for (i = 0; i < SC8562_SCP_RETRY_TIME; i++) {
		/* clear tx/rx fifo */
		ret = sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
			SC8562_SCP_CTRL_CLR_TX_FIFO_MASK, SC8562_SCP_CTRL_CLR_TX_FIFO_SHIFT, 1);
		ret += sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
			SC8562_SCP_CTRL_CLR_RX_FIFO_MASK, SC8562_SCP_CTRL_CLR_RX_FIFO_SHIFT, 1);
		/* write cmd, reg, num */
		ret += sc8562_write_byte(di, SC8562_SCP_WDATA_REG,
			SC8562_SCP_CMD_MBRRD);
		ret += sc8562_write_byte(di, SC8562_SCP_WDATA_REG, reg);
		ret += sc8562_write_byte(di, SC8562_SCP_WDATA_REG, num);

		/* start trans */
		ret += sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
			SC8562_SCP_CTRL_SND_TRANS_MASK,
			SC8562_SCP_CTRL_SND_TRANS_SHIFT, 1);
		if (ret) {
			mutex_unlock(&di->accp_adapter_reg_lock);
			return -EIO;
		}
		/* check cmd transfer success or fail, ignore ack data */
		if (sc8562_scp_cmd_transfer_check(di) == 0) {
			memcpy(val, &di->scp_data[1], num);
			break;
		}
	}
	if (i >= SC8562_SCP_RETRY_TIME) {
		hwlog_err("ic_%u ack error, retry\n", di->ic_role);
		ret = -EINVAL;
	}

	mutex_unlock(&di->accp_adapter_reg_lock);
	return ret;
}

static int sc8562_scp_reg_read(u8 *val, u8 reg, struct sc8562_device_info *di)
{
	int ret;

	if (di->scp_error_flag) {
		hwlog_err("ic_%u scp timeout happened, do not read reg=0x%x\n",
			di->ic_role, reg);
		return -EINVAL;
	}

	ret = sc8562_scp_adapter_reg_read(val, reg, di);
	if (ret) {
		if (reg != HWSCP_ADP_TYPE0)
			di->scp_error_flag = SC8562_SCP_IS_ERR;

		return -EINVAL;
	}

	return 0;
}

static int sc8562_scp_adapter_reg_write(u8 val, u8 reg, struct sc8562_device_info *di)
{
	int ret, i;

	mutex_lock(&di->accp_adapter_reg_lock);

	/* clear scp data */
	memset(di->scp_data, 0, sizeof(di->scp_data));

	di->scp_op = SC8562_SCP_WRITE_OP;
	di->scp_op_num = SC8562_SCP_SBRWR_NUM;
	di->scp_trans_done = false;
	for (i = 0; i < SC8562_SCP_RETRY_TIME; i++) {
		/* clear tx/rx fifo */
		ret = sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
			SC8562_SCP_CTRL_CLR_TX_FIFO_MASK, SC8562_SCP_CTRL_CLR_TX_FIFO_SHIFT, 1);
		ret += sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
			SC8562_SCP_CTRL_CLR_RX_FIFO_MASK, SC8562_SCP_CTRL_CLR_RX_FIFO_SHIFT, 1);
		/* write data */
		ret += sc8562_write_byte(di, SC8562_SCP_WDATA_REG,
			SC8562_SCP_CMD_SBRWR);
		ret += sc8562_write_byte(di, SC8562_SCP_WDATA_REG, reg);
		ret += sc8562_write_byte(di, SC8562_SCP_WDATA_REG, val);
		/* start trans */
		ret += sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
			SC8562_SCP_CTRL_SND_TRANS_MASK,
			SC8562_SCP_CTRL_SND_TRANS_SHIFT, 1);
		if (ret) {
			mutex_unlock(&di->accp_adapter_reg_lock);
			return -EIO;
		}
		/* check cmd transfer success or fail */
		if (sc8562_scp_cmd_transfer_check(di) == 0)
			break;
	}
	if (i >= SC8562_SCP_RETRY_TIME) {
		hwlog_err("ic_%u ack error, retry\n", di->ic_role);
		ret = -EINVAL;
	}

	mutex_unlock(&di->accp_adapter_reg_lock);
	return ret;
}

static int sc8562_scp_adapter_multi_reg_write(u8 reg, int *val,
	u8 num, struct sc8562_device_info *di)
{
	int ret, i, j;

	mutex_lock(&di->accp_adapter_reg_lock);

	/* clear scp data */
	memset(di->scp_data, 0, sizeof(di->scp_data));

	di->scp_op = SC8562_SCP_WRITE_OP;
	di->scp_op_num = num;
	di->scp_trans_done = false;
	for (i = 0; i < SC8562_SCP_RETRY_TIME; i++) {
		/* clear tx/rx fifo */
		ret = sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
			SC8562_SCP_CTRL_CLR_TX_FIFO_MASK, SC8562_SCP_CTRL_CLR_TX_FIFO_SHIFT, 1);
		ret += sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
			SC8562_SCP_CTRL_CLR_RX_FIFO_MASK, SC8562_SCP_CTRL_CLR_RX_FIFO_SHIFT, 1);
		/* write cmd, reg, num */
		ret += sc8562_write_byte(di, SC8562_SCP_WDATA_REG,
			SC8562_SCP_CMD_MBRWR);
		hwlog_info("ic_%u write reg[%x]=0x%x\n",
			di->ic_role, SC8562_SCP_WDATA_REG, SC8562_SCP_CMD_MBRWR);
		ret += sc8562_write_byte(di, SC8562_SCP_WDATA_REG, reg);
		hwlog_info("ic_%u write reg[%x]=0x%x\n",
			di->ic_role, SC8562_SCP_WDATA_REG, reg);
		ret += sc8562_write_byte(di, SC8562_SCP_WDATA_REG, num);
		hwlog_info("ic_%u write reg[%x]=0x%x\n",
			di->ic_role, SC8562_SCP_WDATA_REG, num);

		for (j = 0; j < num; j++) {
			ret += sc8562_write_byte(di, SC8562_SCP_WDATA_REG, (u8)val[j]);
			hwlog_info("ic_%u write reg[%x]=0x%x\n",
				di->ic_role, SC8562_SCP_WDATA_REG, val[j]);
		}
		/* start trans */
		ret += sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
			SC8562_SCP_CTRL_SND_TRANS_MASK,
			SC8562_SCP_CTRL_SND_TRANS_SHIFT, 1);
		if (ret) {
			mutex_unlock(&di->accp_adapter_reg_lock);
			return -EIO;
		}
		/* check cmd transfer success or fail, ignore ack data */
		if (sc8562_scp_cmd_transfer_check(di) == 0) {
			break;
		}
	}
	if (i >= SC8562_SCP_RETRY_TIME) {
		hwlog_err("ic_%u ack error, retry\n", di->ic_role);
		ret = -EINVAL;
	}

	mutex_unlock(&di->accp_adapter_reg_lock);
	return ret;

}

static int sc8562_scp_reg_write(u8 val, u8 reg, struct sc8562_device_info *di)
{
	int ret;

	if (di->scp_error_flag) {
		hwlog_err("ic_%u scp timeout happened, do not write reg=0x%x\n",
			di->ic_role, reg);
		return -EINVAL;
	}

	ret = sc8562_scp_adapter_reg_write(val, reg, di);
	if (ret) {
		di->scp_error_flag = SC8562_SCP_IS_ERR;
		return -EINVAL;
	}

	return 0;
}

static int sc8562_fcp_adapter_vol_check(int adapter_vol, struct sc8562_device_info *di)
{
	int i, ret;
	int adc_vol = 0;

	if ((adapter_vol < SC8562_FCP_ADAPTER_MIN_VOL) ||
		(adapter_vol > SC8562_FCP_ADAPTER_MAX_VOL)) {
		hwlog_err("ic_%u check vol out of range, input vol = %dmV\n",
			di->ic_role, adapter_vol);
		return -EINVAL;
	}

	for (i = 0; i < SC8562_FCP_ADAPTER_VOL_CHECK_TIME; i++) {
		ret = sc8562_get_vbus_mv((unsigned int *)&adc_vol, di);
		if (ret)
			continue;
		if ((adc_vol > (adapter_vol - SC8562_FCP_ADAPTER_ERR_VOL)) &&
			(adc_vol < (adapter_vol + SC8562_FCP_ADAPTER_ERR_VOL)))
			break;
		(void)power_msleep(DT_MSLEEP_100MS, 0, NULL);
	}
	if (i == SC8562_FCP_ADAPTER_VOL_CHECK_TIME) {
		hwlog_err("ic_%u check vol timeout, input vol = %dmV\n",
			di->ic_role, adapter_vol);
		return -EINVAL;
	}

	hwlog_info("ic_%u check vol success, input vol = %dmV, spend %dms\n",
		di->ic_role, adapter_vol, i * DT_MSLEEP_100MS);
	return 0;
}

static int sc8562_fcp_master_reset(void *dev_data)
{
	int ret;
	struct sc8562_device_info *di = dev_data;

	if (!di)
		return -ENODEV;

	ret = sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
		SC8562_SCP_CTRL_SCP_SOFT_RST_MASK, SC8562_SCP_CTRL_SCP_SOFT_RST_SHIFT, 1);
	power_usleep(DT_USLEEP_10MS);

	return ret;
}

static int sc8562_fcp_adapter_reset(void *dev_data)
{
	int ret;
	struct sc8562_device_info *di = dev_data;

	if (!di)
		return -ENODEV;

	ret = sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
		SC8562_SCP_CTRL_SCP_SOFT_RST_MASK, SC8562_SCP_CTRL_SCP_SOFT_RST_SHIFT, 1);
	power_usleep(DT_USLEEP_20MS);
	ret += sc8562_fcp_adapter_vol_check(SC8562_FCP_ADAPTER_RST_VOL, di);
	if (ret)
		return -EINVAL;

	ret = sc8562_config_vusb_ovp_th_mv(di, SC8562_VUSB_VWPC_OVP_INIT);
	ret += sc8562_config_vbus_ovp_th_mv(di, SC8562_VBUS_OVP_FBPSC_BASE, SC8562_CHG_FBYPASS_MODE);

	return ret;
}

static int sc8562_fcp_read_switch_status(void *dev_data)
{
	return 0;
}

static int sc8562_is_fcp_charger_type(void *dev_data)
{
	struct sc8562_device_info *di = dev_data;

	if (!di)
		return 0;

	if (di->dts_fcp_support == 0) {
		hwlog_err("ic_%u not support fcp\n", di->ic_role);
		return 0;
	}

	if (di->fcp_support)
		return 1;

	return 0;
}

static void sc8562_fcp_adapter_detect_reset(struct sc8562_device_info *di)
{
	(void)sc8562_write_mask(di, SC8562_DPDM_CTRL1_REG,
		SC8562_DPDM_EN_MASK, SC8562_DPDM_EN_SHIFT, false);
	(void)sc8562_write_mask(di, SC8562_DPDM_CTRL2_REG,
		SC8562_DP_BUFF_EN_MASK, SC8562_DP_BUFF_EN_SHIFT, false);
	(void)sc8562_write_mask(di, SC8562_SCP_CTRL_REG, SC8562_SCP_CTRL_SCP_EN_MASK,
		SC8562_SCP_CTRL_SCP_EN_SHIFT, false);
	(void)sc8562_fcp_adapter_reset(di);
}

static int sc8562_fcp_adapter_detect_enable(struct sc8562_device_info *di)
{
	int ret;

	ret = sc8562_write_mask(di, SC8562_SCP_CTRL_REG, SC8562_SCP_CTRL_SCP_EN_MASK,
		SC8562_SCP_CTRL_SCP_EN_SHIFT, true);
	ret += sc8562_write_mask(di, SC8562_DPDM_CTRL1_REG, SC8562_DP_20K_PD_EN_MASK,
		SC8562_DP_20K_PD_EN_SHIFT, true);
	ret += sc8562_write_mask(di, SC8562_DPDM_CTRL1_REG, SC8562_DPDM_EN_MASK,
		SC8562_DPDM_EN_SHIFT, true);
	ret += sc8562_write_mask(di, SC8562_DPDM_CTRL2_REG,
		SC8562_DP_BUFF_EN_MASK, SC8562_DP_BUFF_EN_SHIFT, true);
	ret += sc8562_write_byte(di, SC8562_SCP_FLAG_MASK_REG,
		SC8562_SCP_FLAG_MASK_REG_INIT);

	return ret;
}

static int sc8562_fcp_adapter_detct_dpdm_stat(struct sc8562_device_info *di)
{
	int cnt, ret;
	u8 stat = 0;

	for (cnt = 0; cnt < SC8562_SCP_DETECT_MAX_CNT; cnt++) {
		ret = sc8562_read_byte(di, SC8562_DPDM_STAT_REG, &stat);
		hwlog_info("ic_%u scp_dpdm_stat=0x%x\n", di->ic_role, stat);
		if (ret) {
			hwlog_err("ic_%u read dpdm_stat_reg fail\n", di->ic_role);
			continue;
		}

		/* 0: DM voltage < 0.325v, 2: 0.425v < DP voltage < 1v */
		if ((((stat & SC8562_DPDM_STAT_VDM_RD_MASK) >> SC8562_DPDM_STAT_VDM_RD_SHIFT) == 0) &&
			((((stat & SC8562_DPDM_STAT_VDP_RD_MASK) >> SC8562_DPDM_STAT_VDP_RD_SHIFT) == 2)))
			break;
		(void)power_msleep(DT_MSLEEP_100MS, 0, NULL);
	}
	if (cnt == SC8562_SCP_DETECT_MAX_CNT) {
		hwlog_err("ic_%u CHG_SCP_ADAPTER_DETECT_OTHER\n", di->ic_role);
		return -EINVAL;
	}

	return 0;
}

static int sc8562_fcp_adapter_detect_ping_stat(struct sc8562_device_info *di)
{
	int cnt, ret;
	u8 scp_stat = 0;

	for (cnt = 0; cnt < SC8562_SCP_PING_DETECT_MAX_CNT; cnt++) {
		/* wait 82ms for every 5-ping */
		if ((cnt % 5) == 0)
			power_msleep(82, 0, NULL);

		ret = sc8562_write_mask(di, SC8562_SCP_CTRL_REG,
			SC8562_SCP_CTRL_SND_TRANS_MASK,
			SC8562_SCP_CTRL_SND_TRANS_SHIFT, true);
		if (ret)
			return -EIO;

		/* wait 10ms for every ping */
		power_usleep(DT_USLEEP_10MS);

		sc8562_read_byte(di, SC8562_SCP_STAT_REG, &scp_stat);
		hwlog_info("ic_%u scp ping detect,scp_stat:0x%x\n", di->ic_role, scp_stat);
		if ((((scp_stat & SC8562_NO_FIRST_SLAVE_PING_STAT_MASK) >> SC8562_NO_FIRST_SLAVE_PING_STAT_SHIFT) == 0) &&
			(((scp_stat & SC8562_NO_TX_PKT_STAT_MASK) >> SC8562_NO_TX_PKT_STAT_SHIFT) == 1)) {
			hwlog_info("ic_%u scp adapter detect ok\n", di->ic_role);
			di->fcp_support = true;
			break;
		}
	}
	if (cnt == SC8562_SCP_PING_DETECT_MAX_CNT) {
		hwlog_err("ic_%u CHG_SCP_ADAPTER_DETECT_OTHER\n", di->ic_role);
		return -EINVAL;
	}

	return 0;
}

static bool sc8562_fcp_adapter_detect_check(struct sc8562_device_info *di)
{
	u8 dpdm_ctrl = 0;
	u8 scp_ctrl = 0;
	u8 dpdm_val = 0;
	int ret;

	ret = sc8562_read_byte(di, SC8562_DPDM_CTRL1_REG, &dpdm_ctrl);
	ret += sc8562_read_byte(di, SC8562_SCP_CTRL_REG, &scp_ctrl);
	ret += sc8562_read_byte(di, SC8562_DPDM_STAT_REG, &dpdm_val);
	hwlog_info("%s ret=%d dpdm_ctrl=0x%x scp_ctrl=0x%x dpdm_val=0x%x\n", __func__,
		ret, dpdm_ctrl, scp_ctrl, dpdm_val);
	if (!ret && ((dpdm_ctrl & SC8562_DPDM_EN_MASK) == 1) &&
		(((scp_ctrl & SC8562_SCP_CTRL_SCP_EN_MASK) >> SC8562_SCP_CTRL_SCP_EN_SHIFT) == 1) &&
		(dpdm_val == SC8562_VDPDM_DETECT_SUCC_VAL))
		return true;

	return false;
}

static int sc8562_fcp_adapter_detect(struct sc8562_device_info *di)
{
	int ret;
	u8 enable = 0;
	int vbus_uvp;
	int i;

	mutex_lock(&di->scp_detect_lock);
	di->init_finish_flag = true;

	if (di->fcp_support && sc8562_fcp_adapter_detect_check(di)) {
		mutex_unlock(&di->scp_detect_lock);
		return ADAPTER_DETECT_SUCC;
	}

	/* transmit vdp_src_BC1.2 signal */
	ret = sc8562_fcp_adapter_detect_enable(di);
	if (ret) {
		sc8562_fcp_adapter_detect_reset(di);
		mutex_unlock(&di->scp_detect_lock);
		return ADAPTER_DETECT_OTHER;
	}

	/* waiting for hvdcp */
	for (i = 0; i < SC8562_SCP_HVDCP_CNT; i++) {
		vbus_uvp = sc8562_get_vbus_uvp_status(di);
		if (vbus_uvp) {
			hwlog_err("0x%x vbus uv happen, adapter plug out\n", vbus_uvp);
			mutex_unlock(&di->scp_detect_lock);
			return ADAPTER_DETECT_OTHER;
		}
		(void)power_msleep(DT_MSLEEP_100MS, 0, NULL);
	}

	ret = sc8562_read_byte(di, SC8562_SCP_CTRL_REG, &enable);
	if (ret || (((enable & SC8562_SCP_CTRL_SCP_EN_MASK) >> SC8562_SCP_CTRL_SCP_EN_SHIFT) == 0)) {
		hwlog_info("ic_%u scp en detect,scp_en:0x%x\n", di->ic_role, enable);
		sc8562_fcp_adapter_detect_reset(di);
		mutex_unlock(&di->scp_detect_lock);
		return ADAPTER_DETECT_OTHER;
	}

	/* detect dpdm stat */
	ret = sc8562_fcp_adapter_detct_dpdm_stat(di);
	if (ret) {
		sc8562_fcp_adapter_detect_reset(di);
		mutex_unlock(&di->scp_detect_lock);
		return ADAPTER_DETECT_OTHER;
	}

	/* detect ping stat */
	ret = sc8562_fcp_adapter_detect_ping_stat(di);
	if (ret) {
		sc8562_fcp_adapter_detect_reset(di);
		mutex_unlock(&di->scp_detect_lock);
		return ADAPTER_DETECT_OTHER;
	}

	mutex_unlock(&di->scp_detect_lock);
	return ADAPTER_DETECT_SUCC;
}

static int sc8562_fcp_stop_charge_config(void *dev_data)
{
	int ret;
	struct sc8562_device_info *di = dev_data;

	if (!di)
		return -ENODEV;

	ret = sc8562_fcp_master_reset(di);
	ret += sc8562_write_mask(di, SC8562_SCP_CTRL_REG, SC8562_SCP_CTRL_SCP_EN_MASK,
		SC8562_SCP_CTRL_SCP_EN_SHIFT, 0);
	ret += sc8562_write_mask(di, SC8562_DPDM_CTRL1_REG, SC8562_DPDM_EN_MASK,
		SC8562_DPDM_EN_SHIFT, 0);
	if (ret)
		return -EINVAL;

	di->fcp_support = false;

	return ret;
}

static int sc8562_self_check(void *dev_data)
{
	return 0;
}

static int sc8562_scp_chip_reset(void *dev_data)
{
	return sc8562_fcp_master_reset(dev_data);
}

static int sc8562_scp_dpdm_reset(void *dev_data)
{
	struct sc8562_device_info *di = dev_data;

	if (!di)
		return -ENODEV;

	return sc8562_write_mask(di, SC8562_DPDM_CTRL1_REG,
		SC8562_DPDM_EN_MASK, SC8562_DPDM_EN_SHIFT, 0);
}

static int sc8562_scp_reg_read_block(int reg, int *val, int num,
	void *dev_data)
{
	int ret, i;
	u8 data = 0;
	struct sc8562_device_info *di = dev_data;

	if (!di || !val)
		return -ENODEV;

	di->scp_error_flag = SC8562_SCP_NO_ERR;

	for (i = 0; i < num; i++) {
		ret = sc8562_scp_reg_read(&data, reg + i, di);
		if (ret) {
			hwlog_err("ic_%u scp read failed, reg=0x%x\n", di->ic_role, reg + i);
			return -EINVAL;
		}
		val[i] = data;
	}

	return 0;
}

static int sc8562_scp_multi_reg_read_block(u8 reg, u8 *val, u8 num,
	void *dev_data)
{
	int ret;
	u8 *p = val;
	u8 data_len = (num < SC8562_SCP_MULTI_READ_LEN) ? num : SC8562_SCP_MULTI_READ_LEN;

	struct sc8562_device_info *di = dev_data;

	if (!di || !val)
		return -ENODEV;

	di->scp_error_flag = SC8562_SCP_NO_ERR;

	ret = sc8562_scp_adapter_multi_reg_read(reg, p, data_len, di);
	if (ret)
		return -EINVAL;

	num -= data_len;
	if (num) {
		p += data_len;
		reg += data_len;
		ret = sc8562_scp_multi_reg_read_block(reg, p, num, di);
		if (ret)
			return -EINVAL;
	}

	return ret;
}

static int sc8562_scp_multi_reg_write_block(u8 reg, int *val, u8 num,
	void *dev_data)
{
	int ret;
	int *p = val;
	u8 data_len = (num < SC8562_SCP_MULTI_WRITE_LEN) ? num : SC8562_SCP_MULTI_WRITE_LEN;

	struct sc8562_device_info *di = dev_data;

	if (!di || !val)
		return -ENODEV;

	di->scp_error_flag = SC8562_SCP_NO_ERR;

	ret = sc8562_scp_adapter_multi_reg_write(reg, p, data_len, di);
	if (ret)
		return -EINVAL;

	num -= data_len;
	if (num) {
		p += data_len;
		reg += data_len;
		ret = sc8562_scp_multi_reg_write_block(reg, p, num, di);
		if (ret)
			return -EINVAL;
	}

	return ret;
}

static int sc8562_scp_reg_write_block(int reg, const int *val, int num,
	void *dev_data)
{
	int ret, i;
	struct sc8562_device_info *di = dev_data;

	if (!di || !val)
		return -ENODEV;

	di->scp_error_flag = SC8562_SCP_NO_ERR;

	for (i = 0; i < num; i++) {
		ret = sc8562_scp_reg_write(val[i], reg + i, di);
		if (ret) {
			hwlog_err("ic_%u scp write failed, reg=0x%x\n", di->ic_role, reg + i);
			return -EINVAL;
		}
	}

	return 0;
}

static int sc8562_scp_detect_adapter(void *dev_data)
{
	struct sc8562_device_info *di = dev_data;

	if (!di)
		return -ENODEV;

	return sc8562_fcp_adapter_detect(di);
}

static int sc8562_fcp_reg_read_block(int reg, int *val, int num, void *dev_data)
{
	int ret, i;
	u8 data = 0;
	struct sc8562_device_info *di = dev_data;

	if (!di || !val)
		return -ENODEV;

	di->scp_error_flag = SC8562_SCP_NO_ERR;

	for (i = 0; i < num; i++) {
		ret = sc8562_scp_reg_read(&data, reg + i, di);
		if (ret) {
			hwlog_err("ic_%u fcp read failed, reg=0x%x\n", di->ic_role, reg + i);
			return -EINVAL;
		}
		val[i] = data;
	}
	return 0;
}

static int sc8562_fcp_reg_write_block(int reg, const int *val, int num,
	void *dev_data)
{
	int ret, i;
	struct sc8562_device_info *di = dev_data;

	if (!di || !val)
		return -ENODEV;

	di->scp_error_flag = SC8562_SCP_NO_ERR;

	for (i = 0; i < num; i++) {
		ret = sc8562_scp_reg_write(val[i], reg + i, di);
		if (ret) {
			hwlog_err("ic_%u fcp write failed, reg=0x%x\n", di->ic_role, reg + i);
			return -EINVAL;
		}
	}

	return 0;
}

static int sc8562_fcp_detect_adapter(void *dev_data)
{
	struct sc8562_device_info *di = dev_data;

	if (!di)
		return -ENODEV;

	return sc8562_fcp_adapter_detect(di);
}

static int sc8562_pre_init(void *dev_data)
{
	struct sc8562_device_info *di = dev_data;

	if (!di)
		return -ENODEV;

	return sc8562_self_check(di);
}

static int sc8562_scp_adapter_reset(void *dev_data)
{
	return sc8562_fcp_adapter_reset(dev_data);
}

static struct hwscp_ops sc8562_hwscp_ops = {
	.chip_name = "sc8562",
	.reg_read = sc8562_scp_reg_read_block,
	.reg_write = sc8562_scp_reg_write_block,
	.reg_multi_read = sc8562_scp_multi_reg_read_block,
	.reg_multi_write = sc8562_scp_multi_reg_write_block,
	.detect_adapter = sc8562_scp_detect_adapter,
	.soft_reset_master = sc8562_scp_chip_reset,
	.soft_reset_slave = sc8562_scp_adapter_reset,
	.soft_reset_dpdm = sc8562_scp_dpdm_reset,
	.pre_init = sc8562_pre_init,
};

static struct hwfcp_ops sc8562_hwfcp_ops = {
	.chip_name = "sc8562",
	.reg_read = sc8562_fcp_reg_read_block,
	.reg_write = sc8562_fcp_reg_write_block,
	.detect_adapter = sc8562_fcp_detect_adapter,
	.soft_reset_master = sc8562_fcp_master_reset,
	.soft_reset_slave = sc8562_fcp_adapter_reset,
	.get_master_status = sc8562_fcp_read_switch_status,
	.stop_charging_config = sc8562_fcp_stop_charge_config,
	.is_accp_charger_type = sc8562_is_fcp_charger_type,
};

int sc8562_protocol_ops_register(struct sc8562_device_info *di)
{
	int ret;

	if (di->dts_scp_support) {
		sc8562_hwscp_ops.dev_data = (void *)di;
		ret = hwscp_ops_register(&sc8562_hwscp_ops);
		if (ret)
			return -EINVAL;
	}
	if (di->dts_fcp_support) {
		sc8562_hwfcp_ops.dev_data = (void *)di;
		ret = hwfcp_ops_register(&sc8562_hwfcp_ops);
		if (ret)
			return -EINVAL;
	}

	return 0;
}

void sc8562_scp_adapter_default_vset(void *dev_data)
{
	int value[BYTE_ONE];

	value[0] = (5000 - HWSCP_VSSET_OFFSET) / HWSCP_VSSET_STEP; /* set adap default vset 5000 mv */
	sc8562_scp_reg_write_block(HWSCP_VSSET, value, BYTE_ONE, dev_data);
}
